#include <avr/io.h>
#include <avr/interrupt.h>
#include "hal_smb380.h"

#define HAL_SMB380_TRX_CMD_RW         (0x00) /**<  Register Write (short mode). */
#define HAL_SMB380_TRX_CMD_RR         (0x80) /**<  Register Read (short mode). */
#define HAL_SMB380_TRX_CMD_RADDRM     (0x7F) /**<  Register Address Mask. */

 
/*----------------------------------------------------------------------------*/
/** \brief  This function reads the acceleration from one direction.
 *
 *  \param  address Register address of the direction to read from. See datasheet.
 *	\param	*ptr_value Adress where the acceleration value should be stored.
 *	
 *  \returns	1 (true) if the read value is new
 *				0 (false) if the read value is old
 */
int8_t hal_smb380_new_acc_read( uint8_t address, int16_t *ptr_value){
	int16_t low_value=0;
	int16_t high_value=0;
	int16_t value=0;

	uint8_t dummy_read;
	uint8_t dummy_write = 0xFF;

    /* Add the register read command to the address. */
    address &= HAL_SMB380_TRX_CMD_RADDRM;
    address |= HAL_SMB380_TRX_CMD_RR;

	AVR_ENTER_CRITICAL_REGION();

    HAL_SET_CS4_LOW(); /* Start the SPI transaction by pulling the CS4 low. */
	
    /*Send Register address and read 2 register.*/
    SPDR = address;
    while ((SPSR & (1 << SPIF)) == 0) {;}
    dummy_read = SPDR;
    
	SPDR = dummy_write;
    while ((SPSR & (1 << SPIF)) == 0) {;}
    low_value = SPDR;
    
	SPDR = dummy_write;
    while ((SPSR & (1 << SPIF)) == 0) {;}
    high_value = SPDR;
	
    HAL_SET_CS4_HIGH(); /* End the transaction by pulling the CS4 high. */
	
	AVR_LEAVE_CRITICAL_REGION();

	
	/* Check if read data is new */
	if( low_value & 0x0001){
	
		high_value = high_value << 2;
		low_value = low_value >> 6;
	
		value = high_value | low_value;

		/* Convert 10bit signed value in 16bit signed value */
		if( value & 0x0200){
			value = value | 0xFC00;
		}
	
		*ptr_value = value;
	
		return 1;
	}

	return 0;
}
